@@ -1,11 +1,13 @@
 from __future__ import annotations
 
 from dataclasses import dataclass, field
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, TypeVar, Type
 
 if TYPE_CHECKING:
     from tsbot import response
 
+    T = TypeVar("T", bound="TSClientInfo")
+
 
 @dataclass(slots=True)
 class TSClientInfo:
@@ -16,7 +18,7 @@ class TSClientInfo:
     unique_identifier: str = field(compare=True)
 
     @classmethod
-    def from_whoami(cls, resp: response.TSResponse):
+    def from_whoami(cls: Type[T], resp: response.TSResponse) -> T:
         return cls(
             client_id=resp.first["client_id"],
             database_id=resp.first["client_database_id"],
