@@ -1,10 +1,13 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from typing import Iterator
+from typing import TYPE_CHECKING, Generator, Type, TypeVar
 
 from tsbot import utils
 
+if TYPE_CHECKING:
+    T = TypeVar("T", bound="TSResponse")
+
 
 @dataclass(slots=True, frozen=True)
 class TSResponse:
@@ -12,7 +15,7 @@ class TSResponse:
     error_id: int
     msg: str
 
-    def __iter__(self) -> Iterator[dict[str, str]]:
+    def __iter__(self) -> Generator[dict[str, str], None, None]:
         """Iterates through all the datapoints in data"""
         yield from self.data
 
@@ -22,7 +25,7 @@ class TSResponse:
         return self.data[0]
 
     @classmethod
-    def from_server_response(cls, raw_data: list[str]):
+    def from_server_response(cls: Type[T], raw_data: list[str]) -> T:
         error_id, msg = parse_error_line(raw_data.pop())
         data = utils.parse_data("".join(raw_data))
         return cls(data=data, error_id=error_id, msg=msg)
